@@ -215,7 +215,7 @@ class Slit(Waveguide):
             smp.database.save(betas, convs)
         return betas, convs, smp
 
-    def beta(self, w, alpha):
+    def beta(self, w: complex, alpha: tuple[str, int, int]) -> complex:
         """Return phase constant
 
         Args:
@@ -227,6 +227,8 @@ class Slit(Waveguide):
         Returns:
             h: A complex indicating the phase constant.
         """
+        if self.clad.label == "PEC":
+            return self.beta_pec(w, alpha)
         wr = w.real
         wi = w.imag
         hr = self.beta_funcs[(alpha, "real")](wr, wi)[0, 0]
@@ -237,7 +239,7 @@ class Slit(Waveguide):
         #     hi = 1e-16
         return hr + 1j * hi
 
-    def beta_pec(self, w, alpha):
+    def beta_pec(self, w: complex, alpha: tuple[str, int, int]) -> complex:
         """Return phase constant of PEC waveguide
 
         Args:
