@@ -95,6 +95,22 @@ class Sampling(metaclass=abc.ABCMeta):
         )
         return d
 
+    def x(
+        self,
+        h2: complex | np.ndarray,
+        w: complex | np.ndarray,
+        e1: complex | np.ndarray,
+    ) -> complex | np.ndarray:
+        pass
+
+    def y(
+        self,
+        h2: complex | np.ndarray,
+        w: complex | np.ndarray,
+        e2: complex | np.ndarray,
+    ) -> complex | np.ndarray:
+        pass
+
     @abc.abstractmethod
     def u(
         self,
@@ -218,26 +234,39 @@ class Waveguide(metaclass=abc.ABCMeta):
         self.num_m = params["modes"]["num_m"]
         self.bounds = params["bounds"]
         self.ls = params["modes"].get("ls", ["h", "v"])
-        betas, convs, self.samples = self.betas_convs_samples(params)
-        wr_grid, wi_grid = np.meshgrid(self.samples.ws, self.samples.wis, indexing="ij")
-        w_grid = wr_grid + 1j * wi_grid
-        e2s = np.vectorize(self.clad)(w_grid)
-        vs = {}
-        for key in betas.keys():
-            vs[key] = self.samples.v(betas[key] ** 2, w_grid, e2s)
-        self.beta_funcs = self.samples.database.interpolation(
-            betas, convs, vs, self.bounds
-        )
+        if self.clad.label != "PEC":
+            betas, convs, self.samples = self.betas_convs_samples(params)
+            wr_grid, wi_grid = np.meshgrid(
+                self.samples.ws, self.samples.wis, indexing="ij"
+            )
+            w_grid = wr_grid + 1j * wi_grid
+            e2s = np.vectorize(self.clad)(w_grid)
+            vs = {}
+            for key in betas.keys():
+                vs[key] = self.samples.v(betas[key] ** 2, w_grid, e2s)
+            self.beta_funcs = self.samples.database.interpolation(
+                betas, convs, vs, self.bounds
+            )
         alpha_list = []
         alpha_candidates = params["modes"].get("alphas", None)
-        for alpha, comp in self.beta_funcs.keys():
-            if comp == "real":
-                if alpha_candidates is not None:
-                    if alpha in alpha_candidates:
+        if self.clad.label == "PEC":
+            if alpha_candidates is not None:
+                alpha_list = alpha_candidates
+            else:
+                for pol in ["M", "E"]:
+                    for n in range(self.num_n):
+                        for m in range(1, self.num_m + 1):
+                            alpha_list.append((pol, n, m))
+        else:
+            for alpha, comp in self.beta_funcs.keys():
+                if comp == "real":
+                    if alpha_candidates is not None:
+                        if alpha in alpha_candidates:
+                            alpha_list.append(alpha)
+                    else:
                         alpha_list.append(alpha)
-                else:
-                    alpha_list.append(alpha)
         alpha_list.sort()
+
         self.alphas = self.modes = self.get_alphas(alpha_list)
 
         self.alpha_all = [alpha for dir in self.ls for alpha in self.alphas[dir]]
@@ -896,7 +925,7 @@ class Database:
         wl_min = bounds["wl_min"]
         wl_imag = bounds["wl_imag"]
         v_lim = bounds.get("v_lim", None)
-        beta_imag_max = bounds.get("beta_imag_max", None)
+        beta_imag_max = bounds.get("beta_imag_max", 1e7)
         wr_min = 2 * np.pi / wl_max
         wr_max = 2 * np.pi / wl_min
         wi_min = -2 * np.pi / wl_imag
@@ -923,15 +952,15 @@ class Database:
                     vr = vs[alpha][:, ::-1][i_min : i_max + 1, j_min:].real
                     if v_lim is None:
                         vi = vs[alpha][:, ::-1][i_min : i_max + 1, j_min:].imag
-                        v_ok = np.all(np.abs(vr) > np.abs(vi))
+                        v_ok = np.all(np.abs(vr) > np.abs(vi)) & np.all(vr > 0.0)
                     else:
                         v_ok = np.all(vr > v_lim)
                     if np.all(conv[i_min : i_max + 1, j_min:]) and v_ok:
                         data = betas[alpha][:, ::-1][i_min : i_max + 1, j_min:]
-                        if beta_imag_max is not None:
-                            imag_max = data.imag.max()
-                            if imag_max > beta_imag_max:
-                                continue
+                        imag_max = data.imag.max()
+                        data_max = np.abs(data).max()
+                        if imag_max > beta_imag_max or data_max > 1e7:
+                            continue
                         beta_funcs[(alpha, "real")] = RectBivariateSpline(
                             ws[i_min : i_max + 1],
                             wis[j_min:],
